@@ -179,27 +179,25 @@ func (c *Forgejo) Refresh(ctx context.Context, user *model.User) (bool, error) {
 }
 
 // Teams is supported by the Forgejo driver.
-func (c *Forgejo) Teams(ctx context.Context, u *model.User) ([]*model.Team, error) {
+func (c *Forgejo) Teams(ctx context.Context, u *model.User, p *model.ListOptions) ([]*model.Team, error) {
 	client, err := c.newClientToken(ctx, u.AccessToken)
 	if err != nil {
 		return nil, err
 	}
 
-	return shared_utils.Paginate(func(page int) ([]*model.Team, error) {
-		orgs, _, err := client.ListMyOrgs(
-			forgejo.ListOrgsOptions{
-				ListOptions: forgejo.ListOptions{
-					Page:     page,
-					PageSize: c.perPage(ctx),
-				},
+	orgs, _, err := client.ListMyOrgs(
+		forgejo.ListOrgsOptions{
+			ListOptions: forgejo.ListOptions{
+				Page:     p.Page,
+				PageSize: c.perPage(ctx, p.PerPage),
 			},
-		)
-		teams := make([]*model.Team, 0, len(orgs))
-		for _, org := range orgs {
-			teams = append(teams, toTeam(org, c.url))
-		}
-		return teams, err
-	}, -1)
+		},
+	)
+	teams := make([]*model.Team, 0, len(orgs))
+	for _, org := range orgs {
+		teams = append(teams, toTeam(org, c.url))
+	}
+	return teams, err
 }
 
 // TeamPerm is not supported by the Forgejo driver.
@@ -235,23 +233,23 @@ func (c *Forgejo) Repo(ctx context.Context, u *model.User, remoteID model.ForgeR
 
 // Repos returns a list of all repositories for the Forgejo account, including
 // organization repositories.
-func (c *Forgejo) Repos(ctx context.Context, u *model.User) ([]*model.Repo, error) {
+func (c *Forgejo) Repos(ctx context.Context, u *model.User, p *model.ListOptions) ([]*model.Repo, error) {
 	client, err := c.newClientToken(ctx, u.AccessToken)
 	if err != nil {
 		return nil, err
 	}
 
-	repos, err := shared_utils.Paginate(func(page int) ([]*forgejo.Repository, error) {
-		repos, _, err := client.ListMyRepos(
-			forgejo.ListReposOptions{
-				ListOptions: forgejo.ListOptions{
-					Page:     page,
-					PageSize: c.perPage(ctx),
-				},
+	repos, _, err := client.ListMyRepos(
+		forgejo.ListReposOptions{
+			ListOptions: forgejo.ListOptions{
+				Page:     p.Page,
+				PageSize: c.perPage(ctx, p.PerPage),
 			},
-		)
-		return repos, err
-	}, -1)
+		},
+	)
+	if err != nil {
+		return nil, err
+	}
 
 	result := make([]*model.Repo, 0, len(repos))
 	for _, repo := range repos {
@@ -403,7 +401,7 @@ func (c *Forgejo) Deactivate(ctx context.Context, u *model.User, r *model.Repo,
 		hooks, _, err := client.ListRepoHooks(r.Owner, r.Name, forgejo.ListHooksOptions{
 			ListOptions: forgejo.ListOptions{
 				Page:     page,
-				PageSize: c.perPage(ctx),
+				PageSize: c.perPage(ctx, c.pageSize),
 			},
 		})
 		return hooks, err
@@ -676,7 +674,7 @@ func (c *Forgejo) getTagCommitSHA(ctx context.Context, repo *model.Repo, tagName
 	return tag.Commit.SHA, nil
 }
 
-func (c *Forgejo) perPage(ctx context.Context) int {
+func (c *Forgejo) perPage(ctx context.Context, customPerPage int) int {
 	if c.pageSize == 0 {
 		client, err := c.newClientToken(ctx, "")
 		if err != nil {
@@ -689,5 +687,11 @@ func (c *Forgejo) perPage(ctx context.Context) int {
 		}
 		c.pageSize = api.MaxResponseItems
 	}
-	return c.pageSize
+
+	pageSize := customPerPage
+	if pageSize == 0 || pageSize > c.pageSize {
+		pageSize = c.pageSize
+	}
+
+	return pageSize
 }
